/*
								+---------------------------------+
								|                                 |
								|   ***   OGL state guard   ***   |
								|                                 |
								|  Copyright   -tHE SWINe- 2005  |
								|                                 |
								|          OpenGLState.h          |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 05-16-2006
 *
 *  integer used to store just true or false were changed to bool
 *	fixed some minor errors with stencil write-mask
 *  body of CGLState::Update_States moved to TGLState::Update where it belongs
 *
 *	07-15-2006
 *
 *	modified PolygonOffset support to match OpenGL 1.1 (EXT is actualy older)
 *
 *	07-25-2006
 *
 *	Files GlState.cpp and GlState.h renamed to OpenGLState.cpp and OpenGLState.h
 *
 */

#ifndef __GLSTATE_INCLUDED
#define __GLSTATE_INCLUDED

#define __MULTITEXTURE_ENABLED
// undef this part if it's not supposed to run on multitexturing mashine
// (supposedly texturing is so offen it's too costy to check for support every time
// and it's supported on every today's graphics card)

struct __declspec(dllexport) TGLState {
	HGLRC h_glrc;
	// rendering context state is attached to

	int n_references;
	// nomber of references to this state

	GLubyte n_color[4];
	GLfloat f_color[4];
	// color

	GLclampf f_clr_color[4];
	// clear color

	GLclampd f_clr_depth;
	// clear depth

	GLint n_clr_stencil;
	// clear stencil

	bool b_blend;
	GLenum n_blend_func[2]; // src, dest
	// blending

	GLenum n_front_face;
	bool b_cull_face;
	GLenum n_cull_face;
	// face culling

	bool b_alpha_test;
	GLenum n_alpha_func;
	GLclampf f_alpha_tresh;
	// alpha test

	bool b_depth_test;
	GLenum n_depth_func;
	// depth test

	bool b_stencil_test;
	GLenum n_stencil_func[2];
	GLint n_stencil_ref[2];
	GLuint n_stencil_mask[2];
	GLenum n_stencil_op_sfail[2];
	GLenum n_stencil_op_dfail[2];
	GLenum n_stencil_op_dpass[2];
	bool b_separate_gl2_stencil;
	// stencil test

	bool b_texture1d_enabled[32];
	bool b_texture2d_enabled[32];
	bool b_texture3d_enabled[32];
	bool b_texture_cubemap_enabled[32];
	//
	int n_active_tex_unit;
	int n_max_texture_units;
	//
	GLuint n_texture1d[32];
	GLuint n_texture2d[32];
	GLuint n_texture3d[32];
	GLuint n_texture_cubemap[32];
	// texture units

	int n_max_lights;
	bool b_lighting;
	bool b_light[32];
	//
	struct {
		GLfloat f_ambient[4];
		GLfloat f_diffuse[4];
		GLfloat f_specular[4];
		GLfloat f_position[4];
		GLfloat f_spot_direction[3];
		GLfloat f_spot_exponent;
		GLfloat f_spot_cutoff;
		GLfloat f_constant_attenuation;
		GLfloat f_linear_attenuation;
		GLfloat f_quadratic_attenuation;
	} t_light_data[32];
	// lights

	GLfloat f_model_ambient[4];
	GLboolean b_model_local_viewer;
	GLboolean b_model_two_side;
	GLenum n_model_color_control;
	// light model

	GLfloat f_material_ambient[2][4];
	GLfloat f_material_diffuse[2][4];
	GLfloat f_material_specular[2][4];
	GLfloat f_material_emission[2][4];
	GLfloat f_material_shininess[2];
	// materials (0 = back, 1 = front)

	bool b_fog;
	GLenum n_fog_type;
	GLfloat f_fog_color[4];
	GLfloat f_fog_density;
	GLfloat f_fog_start;
	GLfloat f_fog_end;
	// fog

	GLenum n_shade_model;
	// shade model

	GLenum n_polygon_mode[2];
	GLfloat f_line_width;
	GLfloat f_point_size;
	// polygon mode / line width

	bool b_texgen_enabled[32][4];
	GLenum n_texgen_mode[32][4];
	GLfloat f_texgen_eye_plane[32][4][4]; // posledn index jsou data, pedposledn je texcoord a prvn je texture unit
	GLfloat f_texgen_object_plane[32][4][4];
	// texgen

	bool b_depth_write;
	GLuint n_stencil_write[2]; // back / front (back = 0)
	bool b_color_write[4];
	// write masks (stencil mask is real bit-wise mask)

	bool b_polygon_offset_support;
	bool b_polygon_offset[3]; // fill / line / point
	GLfloat f_polygon_offset_bias;
	GLfloat f_polygon_offset_factor;
	// polygon offset

	bool b_vertex_program_support, b_fragment_program_support;
	GLuint n_vertex_program;
	GLuint n_fragment_program;
	bool b_vertex_program_enabled;
	bool b_fragment_program_enabled;
	//
	bool b_shader_support;
	bool b_shader_gl2_support;
	GLuint n_program_object;
	// programmable shading

	bool b_clip_plane[32];
	double f_clip_plane[32][4];
	int n_max_clip_planes;
	// clip planes

	/*
	 *	void TGLState::Update()
	 *		- read current gl-state into itself
	 *		- note it may take a few milliseconds, not a candidate to be called every frame
	 */
	void Update();
};

class __declspec(dllexport) CGLState {
protected:
	static std::vector<TGLState*> *m_p_state;
	static int m_n_state_refs;
	//
	TGLState *m_p_cur_state;
	//
	bool m_b_init;
	//
	HDC m_h_dc;
	HGLRC m_h_glrc;

public:
	CGLState(HDC h_dc, HGLRC h_glrc);
	~CGLState();

	void Update_States();

	HDC h_DeviceContext();
	HGLRC h_RenderingContext();

	const TGLState *p_State() { return m_p_cur_state; }

	void __fastcall Enable(GLenum n_value);
	void __fastcall Disable(GLenum n_value);

	void __fastcall LineWidth(GLfloat f_width);
	void __fastcall PointSize(GLfloat f_size);
	void __fastcall PolygonMode(GLenum n_side, GLenum n_mode);
	// raster props

	void __fastcall EnableFog();
	void __fastcall DisableFog();
	//
	void __fastcall FogMode(GLenum n_value);
	void __fastcall FogDensity(GLfloat f_value);
	void __fastcall FogStart(GLfloat f_value);
	void __fastcall FogEnd(GLfloat f_value);
	void __fastcall FogColor(const GLfloat *p_color);
	// fog

	void __fastcall EnableLighting();
	void __fastcall DisableLighting();
	void __fastcall EnableLight(int n_light); // 0 - 8
	void __fastcall DisableLight(int n_light); // 0 - 8
	//
	void __fastcall ShadeModel(GLenum n_model);
	//
	void __fastcall LightAmbient(int n_light, const GLfloat *p_color4);
	void __fastcall LightDiffuse(int n_light, const GLfloat *p_color4);
	void __fastcall LightSpecular(int n_light, const GLfloat *p_color4);
	void __fastcall LightPosition(int n_light, const GLfloat *p_position4);
	void __fastcall LightSpotDirection(int n_light, const GLfloat *p_direction3);
	void __fastcall LightSpotExponent(int n_light, GLfloat f_value);
	void __fastcall LightSpotCutoff(int n_light, GLfloat f_value);
	void __fastcall LightConstantAttenuation(int n_light, GLfloat f_value);
	void __fastcall LightLinearAttenuation(int n_light, GLfloat f_value);
	void __fastcall LightQuadraticAttenuation(int n_light, GLfloat f_value);
	// lighting

	void __fastcall LightModelAmbient(const GLfloat *p_ambient4);
	void __fastcall LightModelViewer(int b_local);
	void __fastcall LightModelTwoSide(int b_two_side);
	void __fastcall LightModelColorControl(GLenum n_mode);
	// light model

	void __fastcall MaterialAmbient(const GLfloat *p_ambient4);
	void __fastcall MaterialAmbient(int b_front, const GLfloat *p_ambient4);
	void __fastcall MaterialDiffuse(const GLfloat *p_diffuse4);
	void __fastcall MaterialDiffuse(int b_front, const GLfloat *p_diffuse4);
	void __fastcall MaterialSpecular(const GLfloat *p_specular4);
	void __fastcall MaterialSpecular(int b_front, const GLfloat *p_specular4);
	void __fastcall MaterialEmission(const GLfloat *p_emission4);
	void __fastcall MaterialEmission(int b_front, const GLfloat *p_emission4);
	void __fastcall MaterialShininess(GLfloat f_shininess);
	void __fastcall MaterialShininess(int b_front, GLfloat f_shininess);
	// material

	void __fastcall EnableBlend();
	void __fastcall DisableBlend();
	//
	void __fastcall BlendFunc(GLenum n_src_op, GLenum n_dest_op);
	// blending

	void __fastcall EnableDepthTest();
	void __fastcall DisableDepthTest();
	//
	void __fastcall DepthFunc(GLenum n_op);
	// depth test

	void __fastcall EnableStencilTest();
	void __fastcall DisableStencilTest();
	//
	void __fastcall StencilFunc(GLenum n_func, GLint n_ref, GLuint n_mask);
	void __fastcall StencilFunc(bool b_front, GLenum n_func, GLint n_ref, GLuint n_mask);
	void __fastcall StencilOp(GLenum n_s_fail, GLint n_s_pass_d_fail, GLuint n_s_pass_d_pass);
	void __fastcall StencilOp(bool b_front, GLenum n_s_fail, GLint n_s_pass_d_fail, GLuint n_s_pass_d_pass);
	// stencil test

	void __fastcall EnableAlphaTest();
	void __fastcall DisableAlphaTest();
	//
	void __fastcall AlphaFunc(GLenum n_op, GLclampf f_tresh);
	// alpha test

	void __fastcall DisableAllTextures();
	void __fastcall DisableAllTexture1D();
	void __fastcall DisableAllTexture2D();
	void __fastcall DisableAllTexture3D();
	void __fastcall DisableAllTextureCubeMap();
	void __fastcall DisableAllTexture(GLenum n_type);
	//
	void __fastcall EnableTexture1D();
	void __fastcall EnableTexture2D();
	void __fastcall EnableTexture3D();
	void __fastcall EnableTextureCubeMap();
	void __fastcall EnableTexture(GLenum n_type);
	//
	void __fastcall DisableTexture1D();
	void __fastcall DisableTexture2D();
	void __fastcall DisableTexture3D();
	void __fastcall DisableTextureCubeMap();
	void __fastcall DisableTexture(GLenum n_type);
	//
	void __fastcall ActiveTextureUnit(int n_unit); // 0 - 31
	int __fastcall n_TexUnit_Num();
	//
	void __fastcall BindTexture1D(GLuint n_texture);
	void __fastcall BindTexture2D(GLuint n_texture);
	void __fastcall BindTexture3D(GLuint n_texture);
	void __fastcall BindTextureCubeMap(GLuint n_texture);
	void __fastcall BindTexture(GLenum n_type, GLuint n_texture);
	// textures

	void __fastcall Color3f(GLfloat r, GLfloat g, GLfloat b);
	void __fastcall Color4f(GLfloat f, GLfloat g, GLfloat b, GLfloat a);
	//
	void __fastcall Color3b(GLubyte r, GLubyte g, GLubyte b);
	void __fastcall Color4b(GLubyte f, GLubyte g, GLubyte b, GLubyte a);
	// color

	void __fastcall ClearColor4f(GLclampf f, GLclampf g, GLclampf b, GLclampf a);
	void __fastcall ClearDepth(GLclampd d);
	void __fastcall ClearStencil(GLint s);
	// clear attribs

	void __fastcall EnableCullFace();
	void __fastcall DisableCullFace();
	//
	void __fastcall CullFace(GLenum n_mode);
	void __fastcall FrontFace(GLenum n_mode);
	// cull face

	void __fastcall Enable_STRQ_TexGen();
	void __fastcall Disable_STRQ_Texgen();
	//
	void __fastcall Enable_S_TexGen();
	void __fastcall Enable_T_TexGen();
	void __fastcall Enable_R_TexGen();
	void __fastcall Enable_Q_TexGen();
	void __fastcall Disable_S_TexGen();
	void __fastcall Disable_T_TexGen();
	void __fastcall Disable_R_TexGen();
	void __fastcall Disable_Q_TexGen();
	//
	void __fastcall Enable_ST_TexGen();
	void __fastcall Disable_ST_TexGen();
	void __fastcall Enable_STR_TexGen();
	void __fastcall Disable_STR_TexGen();
	void __fastcall EnableTexGen(int n_coord); // 0 = S, 1 = T, 2 = R, 3 = Q
	void __fastcall DisableTexGen(int n_coord); // 0 = S, 1 = T, 2 = R, 3 = Q
	//
	void __fastcall TexGen_STRQ_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_STR_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_S_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_T_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_R_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_Q_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGen_ST_fv(int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	void __fastcall TexGenfv(int n_coord, int n_name, GLfloat *p_param); // name must be one of object plane or eye plane
	//
	void __fastcall TexGen_STRQ_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_STR_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_S_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_T_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_R_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_Q_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGen_ST_Mode(/*int n_name, */GLenum n_param);
	void __fastcall TexGenMode(int n_coord, /*int n_name, */GLenum n_param);
	// texture generation mode

	void __fastcall ColorMask(bool b_r, bool b_g, bool b_b, bool b_a);
	void __fastcall DepthMask(bool b_d);
	void __fastcall StencilMask(GLuint n_mask);
	void __fastcall StencilMask(bool b_front, GLuint n_mask);
	// write masks

	void __fastcall EnablePolygonOffset(GLenum n_mode);
	void __fastcall DisablePolygonOffset(GLenum n_mode);
	void __fastcall EnablePolygonOffset_Line();
	void __fastcall DisablePolygonOffset_Line();
	void __fastcall EnablePolygonOffset_Fill();
	void __fastcall DisablePolygonOffset_Fill();
	void __fastcall EnablePolygonOffset_Point();
	void __fastcall DisablePolygonOffset_Point();
	void __fastcall PolygonOffset(GLfloat f_factor = -1.0f, GLfloat f_bias = -2.0f);
	// polygon offset

	void __fastcall EnableVertexProgram();
	void __fastcall EnableFragmentProgram();
	void __fastcall DisableVertexProgram();
	void __fastcall DisableFragmentProgram();
	//
	void __fastcall BindProgramObject(GLhandleARB h_shader);
	void __fastcall BindProgramObject_GL2(GLuint n_shader);
	void __fastcall BindVertexProgram(GLuint n_program);
	void __fastcall BindFragmentProgram(GLuint n_program);
	// programmable shading

	void __fastcall EnableClipPlane(int n_plane); // 0 .. 6
	void __fastcall DisableClipPlane(int n_plane); // 0 .. 6
	void __fastcall DisableAllClipPlanes();
	//
	void __fastcall ClipPlane(int n_plane, double f_eqn0, double f_eqn1, double f_eqn2, double f_eqn3); // 0 .. 6, normal{x, y, z}, dist
	// clip planes
};

#endif //__GLSTATE_INCLUDED

/*
 *		-end-of-file-
 */
